package dev.fyloz.memorycache

abstract class BaseMemoryCache<K, V>(
    protected val map: MutableMap<K, V> = hashMapOf()
) : MemoryCache<K, V> {
    override fun contains(key: K) =
        key in map

    override fun get(key: K): V? =
        map[key]

    override fun getNotNull(key: K): V {
        if (key !in this) {
            throw KeyNotFoundException(key)
        }

        return get(key)!!
    }

    override fun set(key: K, value: V) {
        map[key] = value
    }

    override fun remove(key: K) {
        map.remove(key)
    }

    override fun clear() {
        map.clear()
    }
}

internal class KeyNotFoundException(val key: Any?) :
    RuntimeException("The cache doesn't contains the requested key '$key'")