package dev.fyloz.memorycache

import java.util.*

private const val defaultCacheCapacity = 1000

/**
 * A basic key-value memory cache with a limited size.
 *
 * When full, the oldest key-value pair will be removed from the cache.
 *
 * @param K The type of the keys.
 * @param V The type of the values.
 * @property capacity The capacity of the cache, which is 1000 by default.
 */
class BasicMemoryCache<K, V>(
    private val capacity: Int = defaultCacheCapacity,
) : BaseMemoryCache<K, V>() {
    private val keys = LinkedList<K>()

    override fun contains(key: K) =
        key in keys

    override fun set(key: K, value: V) {
        cleanup()

        super.set(key, value)
        keys.add(key)
    }

    override fun remove(key: K) {
        super.remove(key)
        keys.remove(key)
    }

    override fun clear() {
        super.clear()
        keys.clear()
    }

    private fun cleanup() {
        if (keys.size < capacity) {
            return
        }

        // Remove the oldest key when the capacity is exceeded, aka the first in the keys linked list
        val oldestKey = keys.pop()
        remove(oldestKey)
    }
}