package dev.fyloz.memorycache

/**
 * Represents a key-value cache whose data is stored in memory.
 *
 * @param K The type of the keys.
 * @param V The type of the values.
 */
interface MemoryCache<K, V> {
    /** Checks if a value correspond to the given [key]. */
    operator fun contains(key: K): Boolean

    /** Gets the value corresponding to the given [key], or null if there is none. */
    operator fun get(key: K): V?

    /** Gets the value corresponding to the given [key]. */
    fun getNotNull(key: K): V

    /** Associates the [value] with the given [key]. The previous value associated with the [key] will be replaced. */
    operator fun set(key: K, value: V)

    /** Removes the value corresponding to the given [key]. */
    fun remove(key: K)

    /** Removes all the cached data. */
    fun clear()
}