package dev.fyloz.memorycache

import io.kotest.core.spec.style.ShouldSpec
import io.kotest.matchers.shouldBe

class BasicMemoryCacheTest : ShouldSpec({
    val cacheCapacity = 20
    val cache = BasicMemoryCache<Int, String>(cacheCapacity)

    val key = Int.MAX_VALUE
    val value = "unit test value"

    afterEach {
        cache.clear()
    }

    context("set") {
        should("add key to internal key list") {
            // Arrange
            // Act
            cache[key] = value

            // Assert
            val contains = key in cache
            contains shouldBe true
        }

        should("remove oldest key-value pair when full") {
            // Arrange
            val firstKey = 0
            val lastKey = firstKey + cacheCapacity

            for (currentKey in firstKey..lastKey) {
                cache[currentKey] = currentKey.toString()
            }

            // Act
            val containsFirstKey = firstKey in cache

            // Assert
            containsFirstKey shouldBe false
        }
    }

    context("remove") {
        should("remove key from internal key list") {
            // Arrange
            cache[key] = value

            // Act
            cache.remove(key)

            // Assert
            val contains = key in cache
            contains shouldBe false
        }
    }

    context("clear") {
        should("remove all keys from internal key list") {
            // Arrange
            cache[key] = value

            // Act
            cache.clear()

            // Assert
            val contains = key in cache
            contains shouldBe false
        }
    }
})